@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2006, 2008, 2011 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2006, 2008, 2011, 2012 Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
    included other routines can be more portable.
 */
 
-#ifndef __CDIO_PORTABLE_H__
-#define __CDIO_PORTABLE_H__
+#ifndef CDIO_DRIVER_PORTABLE_H_
+#define CDIO_DRIVER_PORTABLE_H_
 
 #if defined(HAVE_CONFIG_H) && !defined(__CDIO_CONFIG_H__)
 # include "config.h"
@@ -30,20 +30,20 @@
 #endif
 
 #if !defined(HAVE_FTRUNCATE)
-# if defined ( WIN32 )
+# if defined (_WIN32)
 #  define ftruncate chsize
 # endif
 #endif /*HAVE_FTRUNCATE*/
 
 #if !defined(HAVE_SNPRINTF)
-# if defined ( MSVC )
+# if defined (_MSC_VER)
 #  define snprintf _snprintf
 # endif
 #endif /*HAVE_SNPRINTF*/
 
 #if !defined(HAVE_VSNPRINTF)
-# if defined ( MSVC )
-#  define snprintf _vsnprintf
+# if defined (_MSC_VER)
+#  define vsnprintf _vsnprintf
 # endif
 #endif /*HAVE_SNPRINTF*/
 
@@ -51,26 +51,4 @@
 # define drand48()   (rand() / (double)RAND_MAX)
 #endif
 
-#ifdef MSVC
-# include <io.h>
-
-# ifndef S_ISBLK
-#  define _S_IFBLK        0060000  /* Block Special */
-#  define S_ISBLK(x) (x & _S_IFBLK)
-# endif
-
-# ifndef S_ISCHR
-#  define	_S_IFCHR 0020000	/* character special */
-#  define S_ISCHR(x) (x & _S_IFCHR)
-# endif
-#endif /*MSVC*/
-
-#ifdef HAVE_MEMSET
-# define BZERO(ptr, size) memset(ptr, 0, size)
-#elif  HAVE_BZERO
-# define BZERO(ptr, size) bzero(ptr, size)
-#else 
-#error  You need either memset or bzero
-#endif
-
-#endif /* __CDIO_PORTABLE_H__ */
+#endif /* CDIO_DRIVER_PORTABLE_H_ */
